@@ -47,6 +47,9 @@ thdata::thdata()
   this->dli_equates = false;
   this->d_mark = TT_DATAMARK_TEMP;
   this->d_flags = TT_LEGFLAG_NONE;
+  this->d_vtresh = 67.5;
+  this->d_shape = TT_DATALEG_SHAPE_OCTAGON;
+  this->d_walls = TT_AUTO;
   this->d_last_equate = 0;
   
   this->reset_data_sd();
@@ -74,6 +77,10 @@ thdata::thdata()
   this->stat_st_top = NULL;
   this->stat_st_bottom = NULL;
   
+  this->ugroup = NULL;
+  this->cgroup = this;
+	this->dlu_gradient.allow_percentage = true;
+  
 }
 
 void thdata::reset_data_sd()
@@ -198,13 +205,17 @@ thcmd_option_desc thdata::get_cmd_option_desc(char * opts)
       case TT_DATA_INSTRUMENT:
       case TT_DATA_CALIBRATE:
       case TT_DATA_INFER:
-      case TT_DATA_DATA:
       case TT_DATA_STATION:
       case TT_DATA_EQUATE:
+      case TT_DATA_DATA:
+      case TT_DATA_DECLINATION:
+      case TT_DATA_VTRESH:
         return thcmd_option_desc(id,2);
       default:
         return thcmd_option_desc(id,1);
       case TT_DATA_BREAK:
+      case TT_DATA_GROUP:
+      case TT_DATA_ENDGROUP:
         return thcmd_option_desc(id,0);
     }
 }
@@ -220,36 +231,36 @@ void thdata::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
     case 0:
       // let's split the line
       thsplit_args(& this->db->mbuff_tmp, *args);
-      this->insert_data_leg(this->db->mbuff_tmp.get_size(),
+      this->cgroup->insert_data_leg(this->db->mbuff_tmp.get_size(),
         this->db->mbuff_tmp.get_buffer());
       break;
       
     case TT_DATA_FIX:
       if ((indataline & THOP_INLINE) == 0)
         ththrow(("not a command line option -- fix"))
-      this->set_data_fix(cod.nargs, args);
+      this->cgroup->set_data_fix(cod.nargs, args);
       break;
       
     case TT_DATA_EQUATE:
       if ((indataline & THOP_INLINE) == 0)
         ththrow(("not a command line option -- equate"))
-      this->set_data_equate(cod.nargs, args);
+      this->cgroup->set_data_equate(cod.nargs, args);
       break;
       
     case TT_DATA_FLAGS:
       if ((indataline & THOP_INLINE) == 0)
         ththrow(("not a command line option -- flags"))
-      this->set_data_flags(cod.nargs, args);
+      this->cgroup->set_data_flags(cod.nargs, args);
       break;
       
     case TT_DATA_STATION:
       if ((indataline & THOP_INLINE) == 0)
         ththrow(("not a command line option -- station"))
-      this->set_data_station(cod.nargs, args, argenc);
+      this->cgroup->set_data_station(cod.nargs, args, argenc);
       break;
       
     case TT_DATA_MARK:
-      this->set_data_mark(cod.nargs, args);
+      this->cgroup->set_data_mark(cod.nargs, args);
       break;
 
     case TT_DATA_DATE:
@@ -263,6 +274,26 @@ void thdata::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
       this->discovery_date.join(temp_date);
       break;
       
+    case TT_DATA_WALLS:
+      if (cod.nargs != 1)
+    	  ththrow(("invalid number of option arguments -- walls"));
+      this->cgroup->d_walls = thmatch_token(*args, thtt_onoffauto);
+      if (this->cgroup->d_walls == TT_UNKNOWN_BOOL)
+    	  ththrow(("invalid walls switch -- %s", *args));
+      break;
+      
+    case TT_DATA_SHAPE:
+      if (cod.nargs != 1)
+    	  ththrow(("invalid number of option arguments -- shape"));
+      this->cgroup->d_shape = thmatch_token(*args, thtt_dataleg_shape);
+      if (this->cgroup->d_shape == TT_DATALEG_SHAPE_UNKNOWN)
+    	  ththrow(("unknown shape type -- %s", *args));
+      break;
+      
+    case TT_DATA_VTRESH:
+      this->cgroup->set_data_vtresh(cod.nargs, args);
+      break;
+      
     case TT_DATA_INSTRUMENT:
       if ((indataline & THOP_INLINE) == 0)
         ththrow(("not a command line option -- instrument"))
@@ -272,47 +303,47 @@ void thdata::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
     case TT_DATA_SD:
       if ((indataline & THOP_INLINE) == 0)
         ththrow(("not a command line option -- sd"))
-      this->set_data_sd(cod.nargs, args);
+      this->cgroup->set_data_sd(cod.nargs, args);
       break;
       
     case TT_DATA_DATA:
       if ((indataline & THOP_INLINE) == 0)
         ththrow(("not a command line option -- data"))
-      this->set_data_data(cod.nargs, args);
+      this->cgroup->set_data_data(cod.nargs, args);
       break;
       
     case TT_DATA_INFER:
-      this->set_data_infer(cod.nargs, args);
+      this->cgroup->set_data_infer(cod.nargs, args);
       break;
       
     case TT_DATA_DECLINATION:
-      this->set_data_declination(cod.nargs, args);
+      this->cgroup->set_data_declination(cod.nargs, args);
       break;
       
     case TT_DATA_UNITS:
       if ((indataline & THOP_INLINE) == 0)
         ththrow(("not a command line option -- units"))
-      this->set_data_units(cod.nargs, args);
+      this->cgroup->set_data_units(cod.nargs, args);
       break;
       
     case TT_DATA_GRADE:
       if ((indataline & THOP_INLINE) == 0)
         ththrow(("not a command line option -- grade"))
-      this->set_data_grade(cod.nargs, args);
+      this->cgroup->set_data_grade(cod.nargs, args);
       break;
         
     case TT_DATA_BREAK:
       if (cod.nargs > 0)
         ththrow(("no arguments allowed after break"))
-      this->complete_interleaved_data();
-      this->cd_leg_def = false;
-      this->d_current = 0;
+      this->cgroup->complete_interleaved_data();
+      this->cgroup->cd_leg_def = false;
+      this->cgroup->d_current = 0;
       break;
     
     case TT_DATA_CALIBRATE:
       if ((indataline & THOP_INLINE) == 0)
         ththrow(("not a command line option -- calibrate"))
-      this->set_data_calibration(cod.nargs, args);
+      this->cgroup->set_data_calibration(cod.nargs, args);
       break;
     
     case TT_DATA_TEAM:
@@ -356,6 +387,14 @@ void thdata::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
       this->discovery_team_set.insert(temp_person);
       break;
 
+    case TT_DATA_GROUP:
+      this->start_group();
+      break;
+      
+    case TT_DATA_ENDGROUP:
+      this->end_group();
+      break;
+      
     // if not found, try to set fathers properties  
     default:
       thdataobject::set(cod, args, argenc, indataline);
@@ -423,6 +462,9 @@ void thdata::self_print_properties(FILE * outf)
           fprintf(outf," (nosurvey)\n");
           break;
       }
+      fprintf(outf,"\t\tUDLR: (%g,%g,%g,%g) - (%g,%g,%g,%g)\n",
+        thinn(li->from_up),thinn(li->from_down),thinn(li->from_left),thinn(li->from_right),
+        thinn(li->to_up),thinn(li->to_down),thinn(li->to_left),thinn(li->to_right));
       fprintf(outf,"\t\tdecination: %f deg\n", thinn(li->declination));
       fprintf(outf,"\t\tmark: %d\n", li->s_mark);
       fprintf(outf,"\t\tflags: %d\n", li->flags);
@@ -478,6 +520,16 @@ void thdata::self_print_properties(FILE * outf)
     fprintf(outf," \"%s\" %d (%s [%ld])\n", (si->comment == NULL ? "" : si->comment),
       si->flags, si->srcf.name, si->srcf.line);
   }
+
+  // print dimension lists
+  for(thstdims_list::iterator di = this->dims_list.begin();
+    di != this->dims_list.end(); di++) {
+    fprintf(outf,"\tUDLR: ");
+    fprintf(outf,di->station);
+    fprintf(outf," %g,%g,%g,%g\n", thinn(di->up), thinn(di->down), thinn(di->left), thinn(di->right));
+  }
+
+
 }
 
 
@@ -511,6 +563,7 @@ void thdata::set_data_calibration(int nargs, char ** args)
       case TT_DATALEG_Y:
       case TT_DATALEG_NORTHING:
       case TT_DATALEG_Z:
+      case TT_DATALEG_DIMS:
       case TT_DATALEG_ALTITUDE:
       case TT_DATALEG_BEARING:
       case TT_DATALEG_GRADIENT:
@@ -574,6 +627,24 @@ void thdata::set_data_calibration(int nargs, char ** args)
       case TT_DATALEG_GRADIENT:
         this->dlc_gradient = this->dlc_default;
         break;
+      case TT_DATALEG_DIMS:
+        this->dlc_up = this->dlc_default;
+        this->dlc_down = this->dlc_default;
+        this->dlc_left = this->dlc_default;
+        this->dlc_right = this->dlc_default;
+        break;
+      case TT_DATALEG_UP:
+        this->dlc_up = this->dlc_default;
+        break;
+      case TT_DATALEG_DOWN:
+        this->dlc_down = this->dlc_default;
+        break;
+      case TT_DATALEG_LEFT:
+        this->dlc_left = this->dlc_default;
+        break;
+      case TT_DATALEG_RIGHT:
+        this->dlc_right = this->dlc_default;
+        break;
     }
   }
 }  // data calibration
@@ -583,6 +654,8 @@ void thdata::set_data_units(int nargs, char ** args)
 {
   static int items[THDATA_MAX_ITEMS];
   int nitems, iit, itid, sdtype = 0; // 1-length, 2-angle
+	
+	this->dlu_sdangle.allow_percentage = true;
 
   nitems = nargs - 1;
   if (nitems > THDATA_MAX_ITEMS)
@@ -597,6 +670,7 @@ void thdata::set_data_units(int nargs, char ** args)
       case TT_DATALEG_LENGTH:
       case TT_DATALEG_COUNT:
       case TT_DATALEG_DEPTH:
+      case TT_DATALEG_DEPTHCHANGE:
       case TT_DATALEG_POSITION:
       case TT_DATALEG_X:
       case TT_DATALEG_EASTING:
@@ -604,6 +678,7 @@ void thdata::set_data_units(int nargs, char ** args)
       case TT_DATALEG_NORTHING:
       case TT_DATALEG_Z:
       case TT_DATALEG_ALTITUDE:
+      case TT_DATALEG_DIMS:
       case TT_DATALEG_UP:
       case TT_DATALEG_DOWN:
       case TT_DATALEG_LEFT:
@@ -661,6 +736,7 @@ void thdata::set_data_units(int nargs, char ** args)
         this->dlu_counter = this->dlu_sdlength;
         break;
       case TT_DATALEG_DEPTH:
+      case TT_DATALEG_DEPTHCHANGE:
         this->dlu_depth = this->dlu_sdlength;
         break;
       case TT_DATALEG_X:
@@ -688,15 +764,57 @@ void thdata::set_data_units(int nargs, char ** args)
         break;
       case TT_DATALEG_BEARING:
         this->dlu_bearing = this->dlu_sdangle;
+				if (this->dlu_bearing.get_units() == TT_TFU_PERC)
+					ththrow(("percentage bearing specification not allowed"))
         break;
       case TT_DATALEG_GRADIENT:
         this->dlu_gradient = this->dlu_sdangle;
         break;
+      case TT_DATALEG_DIMS:
+        this->dlu_up = this->dlu_sdlength;
+        this->dlu_down = this->dlu_sdlength;
+        this->dlu_left = this->dlu_sdlength;
+        this->dlu_right = this->dlu_sdlength;
+        break;
+      case TT_DATALEG_UP:
+        this->dlu_up = this->dlu_sdlength;
+        break;
+      case TT_DATALEG_DOWN:
+        this->dlu_down = this->dlu_sdlength;
+        break;
+      case TT_DATALEG_LEFT:
+        this->dlu_left = this->dlu_sdlength;
+        break;
+      case TT_DATALEG_RIGHT:
+        this->dlu_right = this->dlu_sdlength;
+        break;
     }
   }
+
+	this->dlu_sdangle.allow_percentage = false;
 }  // data units
   
 
+void thdata::set_data_vtresh(int nargs, char ** args)
+{
+
+  if ((nargs < 1) || (nargs > 2))
+    ththrow(("invalid number of option arguments -- vtreshold"))
+  int nid;
+  thtfangle dlu;
+  thparse_double(nid, this->d_vtresh, args[0]);
+  if (nid != TT_SV_NUMBER)
+    ththrow(("invalid vtreshold -- %s", args[0]))
+  if (nargs > 1) {
+    dlu.parse_units(args[1]);
+    this->d_vtresh = dlu.transform(this->d_vtresh);
+  }
+  if ((this->d_vtresh < 0.0) || (this->d_vtresh > 90.0))
+    ththrow(("vtreshold out of range -- %s", args[0]))
+  
+}
+
+
 void thdata::set_data_declination(int nargs, char ** args)
 {
   if (nargs > 2)
@@ -908,6 +1026,7 @@ void thdata::set_data_data(int nargs, char ** args)
   for(dix = 1; dix < nargs; dix++) {
     idd = thmatch_token(args[dix], thtt_dataleg_comp);
     switch (idd) {
+
       case TT_DATALEG_STATION:
         if (this->di_station) {
           err_duplicate = true;
@@ -934,6 +1053,10 @@ void thdata::set_data_data(int nargs, char ** args)
           err_inimm = true;
           break;
         }
+        if (this->d_type == TT_DATATYPE_DIMS) {
+          err_itype = true;
+          break;
+        }
         this->di_from = true;
         break;        
      
@@ -946,6 +1069,10 @@ void thdata::set_data_data(int nargs, char ** args)
           err_inimm = true;
           break;
         }
+        if (this->d_type == TT_DATATYPE_DIMS) {
+          err_itype = true;
+          break;
+        }
         this->di_to = true;
         break;
         
@@ -1222,6 +1349,10 @@ void thdata::set_data_data(int nargs, char ** args)
           err_duplicate = true;
           break;
         }
+        if (this->d_type == TT_DATATYPE_DIMS) {
+          err_itype = true;
+          break;
+        }
         this->di_direction = true;
         break;
         
@@ -1230,6 +1361,10 @@ void thdata::set_data_data(int nargs, char ** args)
           err_duplicate = true;
           break;
         }
+        if (this->d_type == TT_DATATYPE_DIMS) {
+          err_itype = true;
+          break;
+        }
         if ((dix == 1) || (dix == nargs - 1))
           ththrow(("invalid newline position")) 
         this->di_newline = true;
@@ -1285,7 +1420,9 @@ void thdata::set_data_data(int nargs, char ** args)
 
       case TT_DATALEG_IGNORE:
         break;
-        
+
+      case TT_DATALEG_IGNOREALL:
+        break;        
         
       default:
         ththrow(("invalid identifier -- %s", args[dix]))
@@ -1300,9 +1437,13 @@ void thdata::set_data_data(int nargs, char ** args)
       ththrow(("invalid reading for this style -- %s", args[dix]))
     if (err_idanl)
       ththrow(("interleaved reading after newline -- %s", args[dix]))
-    // if no errors, set order  
+   
+    // if no errors, no ignoreall - set order  
     this->d_order[dix - 1] = idd;
     this->d_nitems++;
+    if (idd == TT_DATALEG_IGNOREALL)
+      break;
+      
   }
   
   // check if all the data specified
@@ -1327,6 +1468,9 @@ void thdata::set_data_data(int nargs, char ** args)
     case TT_DATATYPE_NOSURVEY:
       all_data = true;
       break;
+    case TT_DATATYPE_DIMS:
+      all_data = this->di_up || this->di_down || this->di_left || this->di_right;
+      break;
   }
   all_data = all_data &&
       ((this->di_from && this->di_to) || this->di_station);
@@ -1336,15 +1480,74 @@ void thdata::set_data_data(int nargs, char ** args)
 } // data data
 
 
+void thdata_parse_dim(char * src, double & d1, double & d2, 
+  bool & d2ok, char * item, thtfpwf * ctran, thtflength * utran) {
+
+  int sv;  
+  d1 = thnan;
+  d2 = thnan;
+  
+  if (!d2ok) {
+    thparse_double(sv, d1, src);
+    if ((sv != TT_SV_NUMBER) && (sv != TT_SV_NAN))
+      ththrow(("invalid %s reading -- %s", item, src))
+  } else {
+    char ** args;
+    long nargs;
+    thsplit_args(& thdb.mbuff_tmp2, src);
+    args = thdb.mbuff_tmp2.get_buffer();
+    nargs = thdb.mbuff_tmp2.get_size();    
+    d2ok = false;
+    if ((nargs < 1) || (nargs > 2))
+      ththrow(("invalid %s reading -- %s", item, src))
+    if (nargs > 1) {
+      d2ok = true;
+      thparse_double(sv, d2, args[1]);
+      if ((sv != TT_SV_NUMBER) && (sv != TT_SV_NAN))
+        ththrow(("invalid %s reading -- %s", item, args[1]))
+    }
+    thparse_double(sv, d1, args[0]);
+    if ((sv != TT_SV_NUMBER) && (sv != TT_SV_NAN))
+      ththrow(("invalid %s reading -- %s", item, args[0]))
+  }
+  
+  if (!thisnan(d1)) {
+    d1 = ctran->evaluate(d1);
+    d1 = utran->transform(d1);
+    if (d1 < 0.0)
+      ththrow(("negative %s reading -- %s", item, src))
+  }
+  if (!thisnan(d2)) {
+    d2 = ctran->evaluate(d2);
+    d2 = utran->transform(d2);
+    if (d2 < 0.0)
+      ththrow(("negative %s reading -- %s", item, src))
+  }
+  
+}
+
+
 void thdata::insert_data_leg(int nargs, char ** args)
 {
-  bool to_clear = true;
+  bool to_clear = true, ftgiven = false, ftorder = false;
   thdataleg dumm;
+  thstdims * cdims = NULL;
+  
+#ifdef THDEBUG
+  thdataleg * cdleg;
+#endif
+
   int carg;
-  if (this->d_current == 0) {
+  
+  if ((this->d_current == 0) && (this->d_type != TT_DATATYPE_DIMS)) {
+  
     this->pd_leg = this->cd_leg;
     this->pd_leg_def = this->cd_leg_def;
     this->cd_leg = this->leg_list.insert(this->leg_list.end(),dumm);
+
+#ifdef THDEBUG
+    cdleg = &(*this->cd_leg);
+#endif
     
     // set all the data
     this->cd_leg->srcf = this->db->csrc;
@@ -1354,6 +1557,10 @@ void thdata::insert_data_leg(int nargs, char ** args)
     this->cd_leg->flags = this->d_flags;
     this->cd_leg->psurvey = this->db->get_current_survey();
     
+    this->cd_leg->walls = this->d_walls;
+    this->cd_leg->vtresh = this->d_vtresh;
+    this->cd_leg->shape = this->d_shape;
+    
     this->cd_leg->length_sd = this->dls_length;
     this->cd_leg->bearing_sd = this->dls_bearing;
     this->cd_leg->gradient_sd = this->dls_gradient;
@@ -1380,21 +1587,49 @@ void thdata::insert_data_leg(int nargs, char ** args)
       to_clear = false;
   }
   
+  if (this->d_type == TT_DATATYPE_DIMS) {
+    // vlozi novu dimension a nastavi pointer na nu
+    cdims = &(*(this->dims_list.insert(this->dims_list.end(), thstdims())));
+    cdims->psurvey = this->db->get_current_survey();
+    cdims->srcf = this->db->csrc;
+    to_clear = false;
+  }
+  
   // now set the data
   bool exit_new_line = false;
   int val_id;
   double val;
+  
+  ftgiven = this->di_from && this->di_to;
+  if (ftgiven) {
+    for(carg = 0; carg < this->d_nitems; carg++) {
+      if (this->d_order[carg] == TT_DATALEG_FROM) {
+        ftorder = true;
+        break;
+      } else if (this->d_order[carg] == TT_DATALEG_TO) {
+        ftorder = false;
+        break;
+      }
+    }
+  }
+  
   for(carg = 0;this->d_current < this->d_nitems;this->d_current++, carg++) {
   
     if ((carg == nargs) && 
-        (this->d_order[this->d_current] != TT_DATALEG_NEWLINE))
+        ((this->d_order[this->d_current] != TT_DATALEG_NEWLINE)
+         && (this->d_order[this->d_current] != TT_DATALEG_IGNOREALL)))
       ththrow(("not enough data readings"))
   
     switch(this->d_order[this->d_current]) {
     
       case TT_DATALEG_STATION:
-        thparse_objectname(this->cd_leg->station, &(this->db->buff_stations),
-          args[carg]);
+        if (this->d_type == TT_DATATYPE_DIMS) {
+          thparse_objectname(cdims->station, &(this->db->buff_stations),
+            args[carg]);
+        } else {
+          thparse_objectname(this->cd_leg->station, &(this->db->buff_stations),
+            args[carg]);
+        }
         break;
         
       case TT_DATALEG_FROM:
@@ -1442,6 +1677,10 @@ void thdata::insert_data_leg(int nargs, char ** args)
           case TT_SV_NUMBER:
             val = this->dlc_bearing.evaluate(val);
             val = this->dlu_bearing.transform(val);
+            if (val < 0.0)
+              val += 360.0;
+            if (val >= 360.0)
+              val -= 360.0;
             if ((val < 0.0) || (val >= 360.0))
               ththrow(("bearing reading out of range -- %s", args[carg]))
             else
@@ -1464,6 +1703,10 @@ void thdata::insert_data_leg(int nargs, char ** args)
           case TT_SV_NUMBER:
             val = this->dlc_bearing.evaluate(val);
             val = this->dlu_bearing.transform(val);
+            if (val < 0.0)
+              val += 360.0;
+            if (val >= 360.0)
+              val -= 360.0;
             if ((val < 0.0) || (val >= 360.0))
               ththrow(("backwards bearing reading out of range -- %s", args[carg]))
             else
@@ -1688,14 +1931,94 @@ void thdata::insert_data_leg(int nargs, char ** args)
         }
         break;
         
+      case TT_DATALEG_IGNOREALL:
+        carg = nargs;
+        to_clear = true;
       case TT_DATALEG_NEWLINE:
         this->d_current++;
         exit_new_line = true;
         break;
+
+      // iba ak specifikovane from a to, tak moozu byt dve
+      // hodnoty - inak nacitavame do to
+        
+      case TT_DATALEG_UP:
+        if (this->d_type == TT_DATATYPE_DIMS) {
+          ftgiven = false;
+          thdata_parse_dim(args[carg], cdims->up, val, ftgiven, "up", 
+            &this->dlc_up, &this->dlu_up);
+        } else {
+          ftgiven = this->di_from && this->di_to;
+          thdata_parse_dim(args[carg], 
+            this->cd_leg->to_up, this->cd_leg->from_up, ftgiven, "up", 
+            &this->dlc_up, &this->dlu_up);
+          if (ftgiven && ftorder) {
+            val = this->cd_leg->to_up;
+            this->cd_leg->to_up = this->cd_leg->from_up;
+            this->cd_leg->from_up = val;
+          }
+        }
+        break;
+        
+      case TT_DATALEG_DOWN:
+        if (this->d_type == TT_DATATYPE_DIMS) {
+          ftgiven = false;
+          thdata_parse_dim(args[carg], cdims->down, val, ftgiven, "down", 
+            &this->dlc_down, &this->dlu_down);
+        } else {
+          ftgiven = this->di_from && this->di_to;
+          thdata_parse_dim(args[carg], 
+            this->cd_leg->to_down, this->cd_leg->from_down, ftgiven, "down", 
+            &this->dlc_down, &this->dlu_down);
+          if (ftgiven && ftorder) {
+            val = this->cd_leg->to_down;
+            this->cd_leg->to_down = this->cd_leg->from_down;
+            this->cd_leg->from_down = val;
+          }
+        }
+        break;
+        
+      case TT_DATALEG_LEFT:
+        if (this->d_type == TT_DATATYPE_DIMS) {
+          ftgiven = false;
+          thdata_parse_dim(args[carg], cdims->left, val, ftgiven, "left", 
+            &this->dlc_left, &this->dlu_left);
+        } else {
+          ftgiven = this->di_from && this->di_to;
+          thdata_parse_dim(args[carg], 
+            this->cd_leg->to_left, this->cd_leg->from_left, ftgiven, "left", 
+            &this->dlc_left, &this->dlu_left);
+          if (ftgiven && ftorder) {
+            val = this->cd_leg->to_left;
+            this->cd_leg->to_left = this->cd_leg->from_left;
+            this->cd_leg->from_left = val;
+          }
+        }
+        break;
+        
+      case TT_DATALEG_RIGHT:
+        if (this->d_type == TT_DATATYPE_DIMS) {
+          ftgiven = false;
+          thdata_parse_dim(args[carg], cdims->right, val, ftgiven, "right", 
+            &this->dlc_right, &this->dlu_right);
+        } else {
+          ftgiven = this->di_from && this->di_to;
+          thdata_parse_dim(args[carg], 
+            this->cd_leg->to_right, this->cd_leg->from_right, ftgiven, "right", 
+            &this->dlc_right, &this->dlu_right);
+          if (ftgiven && ftorder) {
+            val = this->cd_leg->to_right;
+            this->cd_leg->to_right = this->cd_leg->from_right;
+            this->cd_leg->from_right = val;
+          }
+        }
+        break;
         
     }
+    
     if (exit_new_line)
       break;
+      
   }  // end of data setting
   
   if (carg < nargs)
@@ -1705,12 +2028,12 @@ void thdata::insert_data_leg(int nargs, char ** args)
     if (this->di_fromcount && this->di_tocount && 
       (this->cd_leg->fromcounter > this->cd_leg->tocounter))
       ththrow(("negative counter difference"))
-    if ((this->di_gradient || this->di_backgradient) && (this->di_bearing || this->di_backbearing) &&
-      (!(thisnan(this->cd_leg->bearing) 
-      && thisnan(this->cd_leg->backbearing))) &&
-      (((thisinf(this->cd_leg->gradient) != 0) || 
-      (thisinf(this->cd_leg->backgradient) != 0))))
-      ththrow(("compass reading given on plumbed shot"))
+//    if ((this->di_gradient || this->di_backgradient) && (this->di_bearing || this->di_backbearing) &&
+//      (!(thisnan(this->cd_leg->bearing) 
+//      && thisnan(this->cd_leg->backbearing))) &&
+//      (((thisinf(this->cd_leg->gradient) != 0) || 
+//      (thisinf(this->cd_leg->backgradient) != 0))))
+//      ththrow(("compass reading given on plumbed shot"))
     if ((this->di_gradient && this->di_backgradient) && 
       (((thisinf(this->cd_leg->gradient) != 0) &&
       (thisinf(this->cd_leg->backgradient) == 0)) ||
@@ -1720,6 +2043,11 @@ void thdata::insert_data_leg(int nargs, char ** args)
     this->d_current = 0;
     this->complete_interleaved_data();
   }
+
+  if (this->d_type == TT_DATATYPE_DIMS) {
+    this->d_current = 0;
+  }
+  
 }
 
 
@@ -1737,6 +2065,23 @@ void thdata::complete_interleaved_data()
     this->pd_leg->from = this->pd_leg->station;
     this->pd_leg->to = this->cd_leg->station;
   }
+  
+  if (this->di_up) {
+    this->pd_leg->from_up = this->pd_leg->to_up;
+    this->pd_leg->to_up = this->cd_leg->to_up;
+  }
+  if (this->di_down) {
+    this->pd_leg->from_down = this->pd_leg->to_down;
+    this->pd_leg->to_down = this->cd_leg->to_down;
+  }
+  if (this->di_left) {
+    this->pd_leg->from_left = this->pd_leg->to_left;
+    this->pd_leg->to_left = this->cd_leg->to_left;
+  }
+  if (this->di_right) {
+    this->pd_leg->from_right = this->pd_leg->to_right;
+    this->pd_leg->to_right = this->cd_leg->to_right;
+  }
 
   // set the counter
   if (this->di_count) {
@@ -1756,10 +2101,13 @@ void thdata::complete_interleaved_data()
   
   this->pd_leg->topofil = (this->di_count || this->di_fromcount || this->di_tocount);
   this->pd_leg->is_valid = true;
+  
 }
 
 void thdata::start_insert()
 {
+  if (this->cgroup->ugroup != NULL)
+    ththrow(("missing endgroup"))
   this->complete_interleaved_data();
 }
 
@@ -1938,11 +2286,27 @@ void thdata::set_data_flags(int nargs, char ** args)
   
 void thdata::set_data_mark(int nargs, char ** args)
 {
-  if (nargs > 1)
-    ththrow(("too many option arguments -- mark"))
-  this->d_mark = thmatch_token(args[0], thtt_datamark);
-  if (this->d_mark == TT_DATAMARK_UNKNOWN)
-    ththrow(("unknown type of station mark -- %s", args[0]))
+  if (nargs < 1)
+	  ththrow(("missing option arguments -- mark"));
+
+  int tmp_mark = thmatch_token(args[nargs - 1], thtt_datamark);
+	if (tmp_mark == TT_DATAMARK_UNKNOWN) {
+	  ththrow(("unknown type of station mark -- %s", args[nargs - 1]))
+	}
+		
+  if (nargs > 1) {
+		thdatamark dumm;
+		thdatamark_list::iterator it;
+		int mi;
+	  for(mi = 0; mi < (nargs - 1); mi++) {
+    	it = this->mark_list.insert(this->mark_list.end(),dumm);
+      it->mark = tmp_mark;
+	    thparse_objectname(it->station, & this->db->buff_stations, args[mi]);
+      it->srcf = this->db->csrc;
+      it->psurvey = this->db->get_current_survey();
+    } 
+	} else
+	  this->d_mark = tmp_mark;
 }
 
 
@@ -2037,3 +2401,269 @@ void thdata::set_survey_declination()
   dl_survey_declination_on = true;
 }
 
+void thdata::start_group() {
+  thdata * tmp = new thdata;
+
+  tmp->ugroup = this->cgroup;
+  this->cgroup->complete_interleaved_data();
+  this->cgroup->cd_leg_def = false;
+  this->cgroup->d_current = 0;
+  
+  tmp->db = this->db;
+
+  // nastavi secko tak, ako to mame nastavene teraz
+  tmp->dlu_length = this->cgroup->dlu_length;
+  tmp->dlu_counter = this->cgroup->dlu_counter; 
+  tmp->dlu_depth = this->cgroup->dlu_depth; 
+  tmp->dlu_dx = this->cgroup->dlu_dx; 
+  tmp->dlu_dy = this->cgroup->dlu_dy; 
+  tmp->dlu_dz = this->cgroup->dlu_dz;
+  tmp->dlu_x = this->cgroup->dlu_x; 
+  tmp->dlu_y = this->cgroup->dlu_y; 
+  tmp->dlu_z = this->cgroup->dlu_z; 
+  tmp->dlu_sdlength = this->cgroup->dlu_sdlength;
+  
+  tmp->dlu_bearing = this->cgroup->dlu_bearing; 
+  tmp->dlu_gradient = this->cgroup->dlu_gradient; 
+  tmp->dlu_declination = this->cgroup->dlu_declination; 
+  tmp->dlu_sdangle = this->cgroup->dlu_sdangle;
+  
+  tmp->dlc_length = this->cgroup->dlc_length; 
+  tmp->dlc_gradient = this->cgroup->dlc_gradient; 
+  tmp->dlc_bearing = this->cgroup->dlc_bearing; 
+  tmp->dlc_counter = this->cgroup->dlc_counter; 
+  tmp->dlc_depth = this->cgroup->dlc_depth;
+  tmp->dlc_dx = this->cgroup->dlc_dx; 
+  tmp->dlc_dy = this->cgroup->dlc_dy; 
+  tmp->dlc_dz = this->cgroup->dlc_dz; 
+  tmp->dlc_x = this->cgroup->dlc_x; 
+  tmp->dlc_y = this->cgroup->dlc_y; 
+  tmp->dlc_z = this->cgroup->dlc_z; 
+  tmp->dlc_default = this->cgroup->dlc_default;
+    
+  // dls - data standard deviation and declination
+  tmp->dls_length = this->cgroup->dls_length; 
+  tmp->dls_gradient = this->cgroup->dls_gradient; 
+  tmp->dls_bearing = this->cgroup->dls_bearing; 
+  tmp->dls_counter = this->cgroup->dls_counter; 
+  tmp->dls_depth = this->cgroup->dls_depth;
+  tmp->dls_dx = this->cgroup->dls_dx; 
+  tmp->dls_dy = this->cgroup->dls_dy; 
+  tmp->dls_dz = this->cgroup->dls_dz; 
+  tmp->dls_x = this->cgroup->dls_x; 
+  tmp->dls_y = this->cgroup->dls_y; 
+  tmp->dls_z = this->cgroup->dls_z; 
+  tmp->dl_declination = this->cgroup->dl_declination;
+  tmp->dl_survey_declination = this->cgroup->dl_survey_declination;
+    
+  tmp->dli_plumbs = this->cgroup->dli_plumbs; 
+  tmp->dli_equates = this->cgroup->dli_equates; 
+  tmp->dl_direction = this->cgroup->dl_direction;
+
+  // what is inserted
+  tmp->di_station = this->cgroup->di_station; 
+  tmp->di_from = this->cgroup->di_from; 
+  tmp->di_to = this->cgroup->di_to; 
+  tmp->di_length = this->cgroup->di_length; 
+  tmp->di_bearing = this->cgroup->di_bearing; 
+  tmp->di_gradient = this->cgroup->di_gradient;
+  tmp->di_backbearing = this->cgroup->di_backbearing; 
+  tmp->di_backgradient = this->cgroup->di_backgradient;
+  tmp->di_depth = this->cgroup->di_depth; 
+  tmp->di_fromdepth = this->cgroup->di_fromdepth; 
+  tmp->di_todepth = this->cgroup->di_todepth; 
+  tmp->di_depthchange = this->cgroup->di_depthchange; 
+  tmp->di_count = this->cgroup->di_count; 
+  tmp->di_fromcount = this->cgroup->di_fromcount;
+  tmp->di_tocount = this->cgroup->di_tocount; 
+  tmp->di_dx = this->cgroup->di_dx; 
+  tmp->di_dy = this->cgroup->di_dy; 
+  tmp->di_dz = this->cgroup->di_dz; 
+  tmp->di_direction = this->cgroup->di_direction; 
+  tmp->di_newline = this->cgroup->di_newline; 
+  tmp->di_interleaved = this->cgroup->di_interleaved;
+  tmp->di_up = this->cgroup->di_up; 
+  tmp->di_down = this->cgroup->di_down; 
+  tmp->di_left = this->cgroup->di_left; 
+  tmp->di_right = this->cgroup->di_right;
+  
+  tmp->dl_survey_declination_on = this->cgroup->dl_survey_declination_on;
+  
+  tmp->d_type = this->cgroup->d_type;
+  for (int i = 0; i < THDATA_MAX_ITEMS; i++)
+    tmp->d_order[i] = this->cgroup->d_order[i];
+  tmp->d_nitems = this->cgroup->d_nitems;
+  tmp->d_mark = this->cgroup->d_mark;
+  tmp->d_flags = this->cgroup->d_flags;
+  tmp->d_last_equate = this->cgroup->d_last_equate;
+  
+  tmp->d_vtresh = this->cgroup->d_vtresh;
+  tmp->d_walls = this->cgroup->d_walls;
+  tmp->d_shape = this->cgroup->d_shape;
+
+  this->cgroup = tmp;
+}
+
+
+void thdata::end_group() {
+  if (this->cgroup->ugroup == NULL) {
+    ththrow(("endgroup without startgroup"))
+  }
+  thdata * tmp = this->cgroup;
+  this->cgroup = this->cgroup->ugroup;
+  tmp->complete_interleaved_data();
+  
+  // do cgroupu prida vsetky zoznamy ktore existuju
+  // teda stations, fixes, equates a legs
+  // survey data
+  for(thdataleg_list::iterator li = tmp->leg_list.begin();
+    li != tmp->leg_list.end(); li++)
+    if (li->is_valid) {
+      this->cgroup->leg_list.insert(this->cgroup->leg_list.end(), (*li));
+    }
+  // fixed stations
+  for(thdatafix_list::iterator fxi = tmp->fix_list.begin();
+    fxi != tmp->fix_list.end(); fxi++) {
+    this->cgroup->fix_list.insert(this->cgroup->fix_list.end(), (*fxi));
+  }
+	// marks
+	for(thdatamark_list::iterator mi = tmp->mark_list.begin();
+		mi != tmp->mark_list.end(); mi++) {
+    this->cgroup->mark_list.insert(this->cgroup->mark_list.end(), (*mi));
+  }
+  // equates
+  for(thdataequate_list::iterator ei = tmp->equate_list.begin();
+    ei != tmp->equate_list.end(); ei++) {
+    this->cgroup->equate_list.insert(this->cgroup->equate_list.end(), (*ei));
+  }
+  // stations
+  for(thdatass_list::iterator si = tmp->ss_list.begin();
+    si != tmp->ss_list.end(); si++) {
+    this->cgroup->ss_list.insert(this->cgroup->ss_list.end(), (*si));
+  }
+  // dimensions
+  for(thstdims_list::iterator di = tmp->dims_list.begin();
+    di != tmp->dims_list.end(); di++) {
+    this->cgroup->dims_list.insert(this->cgroup->dims_list.end(), (*di));
+  }
+  
+  tmp->self_delete();
+}
+
+
+struct thdatadimrec {
+  double u,d,l,r;
+  thdatadimrec() : u(thnan), d(thnan), l(thnan), r(thnan) {}
+};
+
+
+typedef std::map<long, thdatadimrec> thdatadimmap;
+
+
+#define setdims(sU,sD,sL,sR,tU,tD,tL,tR) {\
+  if (!thisnan(sU)) tU = sU; \
+  if (!thisnan(sD)) tD = sD; \
+  if (!thisnan(sL)) tL = sL; \
+  if (!thisnan(sR)) tR = sR; \
+  }
+
+#define adddims(sU,sD,sL,sR,tU,tD,tL,tR) {\
+  if (thisnan(tU)) tU = sU; \
+  if (thisnan(tD)) tD = sD; \
+  if (thisnan(tL)) tL = sL; \
+  if (thisnan(tR)) tR = sR; \
+  }
+
+
+void thdata::complete_dimensions()
+{
+
+  thdatadimmap dm, idm;
+  thdatadimmap::iterator dmi;
+  thdatadimrec dr;
+  thdataleg_list::iterator li, pli, nli;
+
+  // vytvori explicitny dim map
+  for(thstdims_list::iterator di = this->dims_list.begin();
+    di != this->dims_list.end(); di++) {
+    dr = dm[di->station.id];
+    setdims(di->up,di->down,di->left,di->right,dr.u,dr.d,dr.l,dr.r);
+    dm[di->station.id] = dr;
+  }
+
+#define start_leg_cycle \
+    for(li = this->leg_list.begin(); li != this->leg_list.end(); li++) \
+      if (li->is_valid) {
+
+#define end_leg_cycle }
+  
+  // vytvori implicitny dim map
+  start_leg_cycle
+    dr = idm[li->from.id];
+    setdims(li->from_up, li->from_down, li->from_left, li->from_right, dr.u, dr.d, dr.l, dr.r)
+    idm[li->from.id] = dr;
+    dr = idm[li->to.id];
+    setdims(li->to_up, li->to_down, li->to_left, li->to_right, dr.u, dr.d, dr.l, dr.r)
+    idm[li->to.id] = dr;
+  end_leg_cycle
+  
+
+  // priradi explicitny dim map
+  if (this->dims_list.size() > 0) {
+    start_leg_cycle  
+      dmi = dm.find(li->from.id);
+      if (dmi != dm.end()) adddims(dmi->second.u, dmi->second.d, dmi->second.l, dmi->second.r, li->from_up, li->from_down, li->from_left, li->from_right);
+      dmi = dm.find(li->to.id);
+      if (dmi != dm.end()) adddims(dmi->second.u, dmi->second.d, dmi->second.l, dmi->second.r, li->to_up, li->to_down, li->to_left, li->to_right);
+    end_leg_cycle
+  }
+
+  pli = this->leg_list.end();
+  start_leg_cycle
+    // ak je zadany aspon jeden udaj
+    if ((!thisnan(li->from_up)) || (!thisnan(li->from_down)) ||
+        (!thisnan(li->from_left)) || (!thisnan(li->from_right)) ||
+        (!thisnan(li->to_up)) || (!thisnan(li->to_down)) ||
+        (!thisnan(li->to_left)) || (!thisnan(li->to_right))) {
+      nli = li++;
+      
+      // predchadzajuca zamera
+      if ((pli != this->leg_list.end()) && (pli->to.id == li->from.id))
+        adddims(pli->to_up, pli->to_down, pli->to_left, pli->to_right, li->from_up, li->from_down, li->from_left, li->from_right);
+      if ((pli != this->leg_list.end()) && (pli->from.id == li->from.id))
+        adddims(pli->from_up, pli->from_down, pli->from_left, pli->from_right, li->from_up, li->from_down, li->from_left, li->from_right);
+
+      // impl. dim map        
+      dmi = idm.find(li->from.id);
+      if (dmi != idm.end()) adddims(dmi->second.u, dmi->second.d, dmi->second.l, dmi->second.r, li->from_up, li->from_down, li->from_left, li->from_right);
+      dmi = idm.find(li->to.id);
+      if (dmi != idm.end()) adddims(dmi->second.u, dmi->second.d, dmi->second.l, dmi->second.r, li->to_up, li->to_down, li->to_left, li->to_right);
+      
+      // prev station/next station
+      adddims(li->to_up, li->to_down, li->to_left, li->to_right, li->from_up, li->from_down, li->from_left, li->from_right);
+      adddims(li->from_up, li->from_down, li->from_left, li->from_right, li->to_up, li->to_down, li->to_left, li->to_right);
+      
+      pli = li;
+    }
+  end_leg_cycle
+  
+  // TODO: pre kazdy rez, kde aspon niekde nieco - podoplna chybajuce
+  // a nastavi walls na off kde nic nenajde
+  start_leg_cycle
+    adddims(li->to_down, li->to_up, li->to_right, li->to_left, li->to_up, li->to_down, li->to_left, li->to_right);
+    adddims(li->to_left, li->to_left, li->to_up, li->to_up, li->to_up, li->to_down, li->to_left, li->to_right);
+    adddims(li->from_down, li->from_up, li->from_right, li->from_left, li->from_up, li->from_down, li->from_left, li->from_right);
+    adddims(li->from_left, li->from_left, li->from_up, li->from_up, li->from_up, li->from_down, li->from_left, li->from_right);
+    if ((thisnan(li->to_up)) || (thisnan(li->from_up)))
+      li->walls = TT_FALSE;
+  end_leg_cycle
+  
+
+}
+
+
+
+
+
+
+
